@@ -1,8 +1,16 @@
 (*---------------------------------------------------------------------------
    Copyright (c) 2008 The uuidm programmers. All rights reserved.
-   Distributed under the ISC license, see terms at the end of the file.
+   SPDX-License-Identifier: ISC
   ---------------------------------------------------------------------------*)
 
+(* Bits *)
+
+type bits62 = int64
+type bits4 = int
+type bits12 = int
+
+(* Hashing *)
+
 let md5 = Digest.string
 let sha_1 s =
   (* Based on pseudo-code of RFC 3174. Slow and ugly but does the job. *)
@@ -16,15 +24,15 @@ let sha_1 s =
     Bytes.fill m len (mlen - len) '\x00';
     Bytes.set m len '\x80';
     if Sys.word_size > 32 then begin
-      Bytes.set m (mlen - 8) (Char.unsafe_chr (blen lsr 56 land 0xFF));
-      Bytes.set m (mlen - 7) (Char.unsafe_chr (blen lsr 48 land 0xFF));
-      Bytes.set m (mlen - 6) (Char.unsafe_chr (blen lsr 40 land 0xFF));
-      Bytes.set m (mlen - 5) (Char.unsafe_chr (blen lsr 32 land 0xFF));
+      Bytes.set_uint8 m (mlen - 8) (blen lsr 56 land 0xFF);
+      Bytes.set_uint8 m (mlen - 7) (blen lsr 48 land 0xFF);
+      Bytes.set_uint8 m (mlen - 6) (blen lsr 40 land 0xFF);
+      Bytes.set_uint8 m (mlen - 5) (blen lsr 32 land 0xFF);
     end;
-    Bytes.set m (mlen - 4) (Char.unsafe_chr (blen lsr 24 land 0xFF));
-    Bytes.set m (mlen - 3) (Char.unsafe_chr (blen lsr 16 land 0xFF));
-    Bytes.set m (mlen - 2) (Char.unsafe_chr (blen lsr 8 land 0xFF));
-    Bytes.set m (mlen - 1) (Char.unsafe_chr (blen land 0xFF));
+    Bytes.set_uint8 m (mlen - 4) (blen lsr 24 land 0xFF);
+    Bytes.set_uint8 m (mlen - 3) (blen lsr 16 land 0xFF);
+    Bytes.set_uint8 m (mlen - 2) (blen lsr 8 land 0xFF);
+    Bytes.set_uint8 m (mlen - 1) (blen land 0xFF);
     m
   in
   (* Operations on int32 *)
@@ -34,7 +42,6 @@ let sha_1 s =
   let ( land ) = Int32.logand in
   let ( ++ ) = Int32.add in
   let lnot = Int32.lognot in
-  let sr = Int32.shift_right in
   let sl = Int32.shift_left in
   let cls n x = (sl x n) lor (Int32.shift_right_logical x (32 - n)) in
   (* Start *)
@@ -53,13 +60,7 @@ let sha_1 s =
   for i = 0 to ((Bytes.length m) / 64) - 1 do              (* For each block *)
     (* Fill w *)
     let base = i * 64 in
-    for j = 0 to 15 do
-      let k = base + (j * 4) in
-      w.(j) <- sl (Int32.of_int (Char.code @@ Bytes.get m k)) 24 lor
-               sl (Int32.of_int (Char.code @@ Bytes.get m (k + 1))) 16 lor
-               sl (Int32.of_int (Char.code @@ Bytes.get m (k + 2))) 8 lor
-               (Int32.of_int (Char.code @@ Bytes.get m (k + 3)))
-    done;
+    for j = 0 to 15 do w.(j) <- Bytes.get_int32_be m (base + (j * 4)); done;
     (* Loop *)
     a := !h0; b := !h1; c := !h2; d := !h3; e := !h4;
     for t = 0 to 79 do
@@ -95,12 +96,7 @@ let sha_1 s =
     h4 := !h4 ++ !e
   done;
   let h = Bytes.create 20 in
-  let i2s h k i =
-    Bytes.set h k (Char.unsafe_chr ((Int32.to_int (sr i 24)) &&& 0xFF));
-    Bytes.set h (k + 1) (Char.unsafe_chr ((Int32.to_int (sr i 16)) &&& 0xFF));
-    Bytes.set h (k + 2) (Char.unsafe_chr ((Int32.to_int (sr i 8)) &&& 0xFF));
-    Bytes.set h (k + 3) (Char.unsafe_chr ((Int32.to_int i) &&& 0xFF));
-  in
+  let i2s h k i = Bytes.set_int32_be h k i in
   i2s h 0 !h0;
   i2s h 4 !h1;
   i2s h 8 !h2;
@@ -110,84 +106,112 @@ let sha_1 s =
 
 (* Uuids *)
 
-type t = string (* 16 bytes long strings *)
+type t = string (* 16 bytes *)
 
-let msg_uuid v digest ns n =
-  let u = Bytes.sub (Bytes.unsafe_of_string (digest (ns ^ n))) 0 16 in
-  Bytes.set u 6 @@ Char.unsafe_chr
-    ((v lsl 4) lor (Char.code (Bytes.get u 6) land 0b0000_1111));
-  Bytes.set u 8 @@ Char.unsafe_chr
-    (0b1000_0000 lor (Char.code (Bytes.get u 8) land 0b0011_1111));
+let make u ~version =
+  let b6 = (version lsl 4) lor ((Bytes.get_uint8 u 6) land 0b0000_1111) in
+  let b8 = 0b1000_0000 lor ((Bytes.get_uint8 u 8) land 0b0011_1111) in
+  Bytes.set_uint8 u 6 b6;
+  Bytes.set_uint8 u 8 b8;
   Bytes.unsafe_to_string u
 
-let v3 ns n = msg_uuid 3 md5 ns n
-let v5 ns n = msg_uuid 5 sha_1 ns n
-let v4 b =
-  let u = Bytes.sub b 0 16 in
-  let b6 = 0b0100_0000 lor (Char.code (Bytes.get u 6) land 0b0000_1111) in
-  let b8 = 0b1000_0000 lor (Char.code (Bytes.get u 8) land 0b0011_1111) in
-  Bytes.set u 6 (Char.unsafe_chr b6);
-  Bytes.set u 8 (Char.unsafe_chr b8);
-  Bytes.unsafe_to_string u
+let make_named ~version digest ns n =
+  let hash = Bytes.unsafe_of_string (digest (ns ^ n)) in
+  make (Bytes.sub hash 0 16) ~version
 
-let rand s = fun () -> Random.State.bits s (* 30 random bits generator. *)
-let default_rand = rand (Random.State.make_self_init ())
-let v4_ocaml_random_uuid rand =
-  let r0 = rand () in
-  let r1 = rand () in
-  let r2 = rand () in
-  let r3 = rand () in
-  let r4 = rand () in
+let v3 ns n = make_named ~version:3 md5 ns n
+let v5 ns n = make_named ~version:5 sha_1 ns n
+let v4 b = make (Bytes.sub b 0 16) ~version:4
+let v7 ~time_ms ~rand_a ~rand_b =
   let u = Bytes.create 16 in
-  Bytes.set u 0 @@ Char.unsafe_chr (r0 land 0xFF);
-  Bytes.set u 1 @@ Char.unsafe_chr (r0 lsr 8 land 0xFF);
-  Bytes.set u 2 @@ Char.unsafe_chr (r0 lsr 16 land 0xFF);
-  Bytes.set u 3 @@ Char.unsafe_chr (r1 land 0xFF);
-  Bytes.set u 4 @@ Char.unsafe_chr (r1 lsr 8 land 0xFF);
-  Bytes.set u 5 @@ Char.unsafe_chr (r1 lsr 16 land 0xFF);
-  Bytes.set u 6 @@ Char.unsafe_chr
-    (0b0100_0000 lor (r1 lsr 24 land 0b0000_1111));
-  Bytes.set u 7 @@ Char.unsafe_chr (r2 land 0xFF);
-  Bytes.set u 8 @@ Char.unsafe_chr
-    (0b1000_0000 lor (r2 lsr 24 land 0b0011_1111));
-  Bytes.set u 9 @@ Char.unsafe_chr (r2 lsr 8 land 0xFF);
-  Bytes.set u 10 @@ Char.unsafe_chr (r2 lsr 16 land 0xFF);
-  Bytes.set u 11 @@ Char.unsafe_chr (r3 land 0xFF);
-  Bytes.set u 12 @@ Char.unsafe_chr (r3 lsr 8 land 0xFF);
-  Bytes.set u 13 @@ Char.unsafe_chr (r3 lsr 16 land 0xFF);
-  Bytes.set u 14 @@ Char.unsafe_chr (r4 land 0xFF);
-  Bytes.set u 15 @@ Char.unsafe_chr (r4 lsr 8 land 0xFF);
-  Bytes.unsafe_to_string u
+  Bytes.set_int64_be u 0 (Int64.shift_left time_ms 16);
+  Bytes.set_int16_be u 6 rand_a;
+  Bytes.set_int64_be u 8 rand_b;
+  make u ~version:7
 
-let v4_gen seed =
-  let rand = rand seed in
-  function () -> v4_ocaml_random_uuid rand
+let v7_ns ~time_ns ~rand_b =
+  let ns_in_ms = 1_000_000L in
+  let sub_ms_frac_multiplier = Int64.unsigned_div Int64.minus_one ns_in_ms in
+  let u = Bytes.create 16 in
+  (* RFC 9562 requires we use 48 bits for a timestamp in milliseconds, and
+     allows for 12 bits to store a sub-millisecond fraction. We get the
+     latter by multiplying to put the fraction in a 64-bit range, then
+     shifting into 12 bits. *)
+  let ms = Int64.unsigned_div time_ns ns_in_ms in
+  let ns = Int64.unsigned_rem time_ns ns_in_ms in
+  let sub_ms_frac =
+    Int64.shift_right_logical (Int64.mul ns sub_ms_frac_multiplier) 52
+  in
+  Bytes.set_int64_be u 0 (Int64.shift_left ms 16);
+  Bytes.set_int16_be u 6 (Int64.to_int sub_ms_frac);
+  Bytes.set_int64_be u 8 rand_b;
+  make u ~version:7
 
-type version = [ `V3 of t * string | `V4 | `V5 of t * string ]
-let v = function
-| `V4 -> v4_ocaml_random_uuid default_rand
-| `V3 (ns, n) -> v3 ns n
-| `V5 (ns, n) -> v5 ns n
+let v8 s =
+  let l = String.length s in
+  if l = 16 then make (Bytes.of_string s) ~version:8 else
+  invalid_arg (Printf.sprintf "expected 16 bytes but found: %d" l)
 
-let create = v (* deprecated *)
+(* Generators *)
+
+type posix_ms_clock = unit -> int64
+
+let v4_random rstate =
+  let r0 = Random.State.bits64 rstate in
+  let r1 = Random.State.bits64 rstate in
+  let u = Bytes.create 16 in
+  Bytes.set_int64_be u 0 r0;
+  Bytes.set_int64_be u 8 r1;
+  make u ~version:4
+
+let v4_gen rstate = function () -> v4_random rstate
+
+let v7_non_monotonic_gen ~now_ms rstate =
+  fun () ->
+  let time_ms = now_ms () in
+  let rand_a = Random.State.bits (* 30 bits *) rstate  in
+  let rand_b = Random.State.bits64 rstate in
+  v7 ~time_ms ~rand_a ~rand_b
+
+let v7_monotonic_gen ~now_ms rstate =
+  let last_ms = ref 0L in
+  let count = ref 0 in
+  fun () ->
+    let time_ms = now_ms () in
+    let rand_b = Random.State.bits64 rstate in
+    if Int64.equal time_ms !last_ms then
+      let rand_a = incr count; !count in
+      if rand_a >= 4096 then None else Some (v7 ~time_ms ~rand_a ~rand_b)
+    else
+      (count := 0; last_ms := time_ms; Some (v7 ~time_ms ~rand_a:0 ~rand_b))
 
 (* Constants *)
 
 let nil = "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
+let max = "\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff"
 let ns_dns = "\x6b\xa7\xb8\x10\x9d\xad\x11\xd1\x80\xb4\x00\xc0\x4f\xd4\x30\xc8"
 let ns_url = "\x6b\xa7\xb8\x11\x9d\xad\x11\xd1\x80\xb4\x00\xc0\x4f\xd4\x30\xc8"
 let ns_oid = "\x6b\xa7\xb8\x12\x9d\xad\x11\xd1\x80\xb4\x00\xc0\x4f\xd4\x30\xc8"
 let ns_X500 ="\x6b\xa7\xb8\x14\x9d\xad\x11\xd1\x80\xb4\x00\xc0\x4f\xd4\x30\xc8"
 
-(* Comparing *)
+(* Properties *)
 
-let equal u u' = (compare : string -> string -> int) u u' = 0
-let compare : string -> string -> int = Stdlib.compare
+let variant u = (String.get_uint8 u 8) lsr 4
+let version u = (String.get_uint8 u 6) lsr 4
+let time_ms u =
+  let variant = variant u in
+  if not (0x8 <= variant && variant <= 0xB && version u = 7) then None else
+  Some (Int64.shift_right_logical (String.get_int64_be u 0) 16)
+
+(* Predicates and comparisons *)
+
+let equal = String.equal
+let compare = String.compare
 
 (* Standard binary format *)
 
-let to_bytes s = s
-let of_bytes ?(pos = 0) s =
+let to_binary_string s = s
+let of_binary_string ?(pos = 0) s =
   let len = String.length s in
   if pos + 16 > len then None else
   if pos = 0 && len = 16 then Some s else
@@ -206,15 +230,14 @@ let mixed_swaps s =
   swap b 4 5; swap b 6 7;
   Bytes.unsafe_to_string b
 
-let to_mixed_endian_bytes s = mixed_swaps s
-let of_mixed_endian_bytes ?pos s = match of_bytes ?pos s with
-| None -> None
-| Some s -> Some (mixed_swaps s)
+let to_mixed_endian_binary_string s = mixed_swaps s
+let of_mixed_endian_binary_string ?pos s =
+  Option.map mixed_swaps (of_binary_string ?pos s)
 
 (* Unsafe conversions *)
 
-let unsafe_of_bytes u = u
-let unsafe_to_bytes u = u
+let unsafe_of_binary_string u = u
+let unsafe_to_binary_string u = u
 
 (* US-ASCII format *)
 
@@ -276,21 +299,24 @@ let to_string ?(upper = false) u =
 (* Pretty-printing *)
 
 let pp ppf u = Format.pp_print_string ppf (to_string u)
+let pp' ~upper ppf u = Format.pp_print_string ppf (to_string ~upper u)
+
+(* Deprecated *)
+
+let default_seed = lazy (Random.State.make_self_init ())
+
+type version = [ `V3 of t * string | `V4 | `V5 of t * string ]
+let v = function
+| `V4 -> v4_random (Lazy.force default_seed)
+| `V3 (ns, n) -> v3 ns n
+| `V5 (ns, n) -> v5 ns n
+
+let create = v (* deprecated *)
 let pp_string ?upper ppf u = Format.pp_print_string ppf (to_string ?upper u)
 let print = pp_string (* deprecated *)
-
-(*---------------------------------------------------------------------------
-   Copyright (c) 2008 The uuidm programmers
-
-   Permission to use, copy, modify, and/or distribute this software for any
-   purpose with or without fee is hereby granted, provided that the above
-   copyright notice and this permission notice appear in all copies.
-
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
-   WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-   MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
-   ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
-   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-  ---------------------------------------------------------------------------*)
+let to_bytes = to_binary_string
+let of_bytes = of_binary_string
+let to_mixed_endian_bytes = to_mixed_endian_binary_string
+let of_mixed_endian_bytes = of_mixed_endian_binary_string
+let unsafe_of_bytes = unsafe_of_binary_string
+let unsafe_to_bytes = unsafe_to_binary_string
