@@ -53,7 +53,7 @@ val v7 : time_ms:int64 -> rand_a:bits12 -> rand_b:bits62 -> t
 (** [v7 ~time_ms ~rand_a ~rand_b] is a
     {{:https://www.rfc-editor.org/rfc/rfc9562#name-uuid-version-7}V7 UUID}
     (time and random based) using the 64-bit millisecond POSIX timestamp
-    [t_ms] and random bits [rand_a] and [rand_b]. See also {!v7_ns},
+    [time_ms] and random bits [rand_a] and [rand_b]. See also {!v7_ns},
     {!v7_non_monotonic_gen} and {!v7_monotonic_gen}.
 
     {b Warning.} The timestamp and the randomness are seen literally
@@ -73,7 +73,7 @@ val v7_ns : time_ns:int64 -> rand_b:bits62 -> t
 val v8 : string -> t
 (** [v8 s] is a {{:https://www.rfc-editor.org/rfc/rfc9562#name-uuid-version-8}
     V8 UUID} (custom) that uses the 16 bytes of [s] but overwrites the
-    {!version} and {!variant} bytes to make it a propert V8 UUID. Raises
+    {!version} and {!variant} bits to make it a propert V8 UUID. Raises
     [Invalid_argument] if the length of [s] is not [16]. *)
 
 (** {1:gen Generators}
@@ -223,11 +223,10 @@ val pp' : upper:bool -> Format.formatter -> t -> unit
 
 (** {1:deprecated Deprecated} *)
 
-type version =
+type[@ocaml.deprecated "Use the version specific Uuidm.v* functions."] version =
 [ `V3 of t * string (** Name based with MD5 hashing *)
 | `V4 (** Random based *)
 | `V5 of t * string (** Name based with SHA-1 hasing *) ]
-[@ocaml.deprecated "Use the version specific Uuidm.v* functions."]
 (** The type for UUID versions and generation parameters.
     {ul
     {- [`V3] and [`V5] specify a namespace and a name for the generation.}
@@ -238,7 +237,7 @@ type version =
        {b Warning.} The sequence resulting from repeatedly calling
        [v `V4] is random but predictable see {!v4_gen}.}} *)
 
-[@@ocaml.warning "-3"]
+[@@@alert "-deprecated"]
 
 val v : version -> t
 [@@ocaml.deprecated "Use the version specific Uuidm.v* functions."]
@@ -271,5 +270,3 @@ val unsafe_of_bytes : string -> t
 val unsafe_to_bytes : t -> string
 [@@ocaml.deprecated "Use Uuidm.unsafe_to_binary_string instead"]
 (**/**)
-
-[@@ocaml.warning "-3"]
